/*
 *
*/

/* Includes ------------------------------------------------------------------*/
#include "stm32f4xx.h"
#include "stm32f4_discovery.h"
#include <stdio.h>
#include <string.h>

/* Private variables ---------------------------------------------------------*/
USART_InitTypeDef USART_InitStructure;

/* Variaveis globais ---------------------------------------------------------*/
int k=0;
char ch[800]={0};
int estado=0;
int estado_imp=0;
char rx[800]={0};
int connect=0;
int status=0;
int cont_erro=0;

/* Private function prototypes -----------------------------------------------*/
#ifdef __GNUC__
  /* With GCC/RAISONANCE, small printf (option LD Linker->Libraries->Small printf
     set to 'Yes') calls __io_putchar() */
  #define PUTCHAR_PROTOTYPE int __io_putchar(int ch)
#else
  #define PUTCHAR_PROTOTYPE int fputc(int ch, FILE *f)
#endif /* __GNUC__ */
  
/* Private functions ---------------------------------------------------------*/
void enviaTX6(char envia[], int str_len,int timing_delay);
int recebeRX6(char str_conf[],int len_conf);
void apagaRecebe();

/*
 * DELAY EM MS
*/
void delay(uint32_t ms){
  ms *= 3360;
  while(ms--){
    __NOP();
  }
}

/*
 * DIVIDIR STRINGS COM O NUMERO DE POSIES OU CARACTER DE PARAGEM
*/
void substring(char s[],char sub[],int p,int l,char ch){
  int c=0;
  while(c<l && s[p+c-1]!=ch) 
  {
    sub[c]=s[p+c-1];
    c++;
  }
  sub[c-1]='\0';
}

/*
 * APAGAR O ARRAY DE RECECAO DA UART
*/
void apagaRecebe(){
  int l;
  for(l=0;l<800;l++){
    ch[l]=0;
  }
}

/*
 * CONFIGURAR O PORTO DO LED
*/
void CONFIG_LED(){
  RCC->AHB1ENR |= (0x01 << 3);
  GPIOD->MODER &= ~((0x03<<30) | (0x03<<28) | (0x03<<26) | (0x03<<24));
  GPIOD->MODER |= ((0x01<<30) | (0x01<<28) | (0x01<<26) | (0x01<<24));
  GPIOD->OTYPER &= ~((0x01<<15) | (0x01<<14) | (0x01<<13) | (0x01<<12));
  GPIOD->BSRRH |= ((0x01<<15) | (0x01<<14) | (0x01<<13) | (0x01<<12));
}

/*
 * CONFIGURAR UART1
*/
void CONFIG_UART1(){
  /* USARTx configured as follow:
        - BaudRate = 115200 baud  
        - Word Length = 8 Bits
        - One Stop Bit
        - No parity
        - Hardware flow control disabled (RTS and CTS signals)
        - Receive and transmit enabled
  */
  GPIO_InitTypeDef GPIO_InitStruct; // this is for the GPIO pins used as TX and RX
  USART_InitTypeDef USART_InitStruct; // this is for the USART1 initilization
  //NVIC_InitTypeDef NVIC_InitStructure; // this is used to configure the NVIC (nested vector interrupt controller)
  
  RCC_APB2PeriphClockCmd(RCC_APB2Periph_USART1, ENABLE);
  RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOB, ENABLE);
  
  GPIO_InitStruct.GPIO_Pin = GPIO_Pin_6 | GPIO_Pin_7; // Pins 6 (TX) and 7 (RX) are used
  GPIO_InitStruct.GPIO_Mode = GPIO_Mode_AF; 			// the pins are configured as alternate function so the USART peripheral has access to them
  GPIO_InitStruct.GPIO_Speed = GPIO_Speed_50MHz;		// this defines the IO speed and has nothing to do with the baudrate!
  GPIO_InitStruct.GPIO_OType = GPIO_OType_PP;			// this defines the output type as push pull mode (as opposed to open drain)
  GPIO_InitStruct.GPIO_PuPd = GPIO_PuPd_UP;			// this activates the pullup resistors on the IO pins
  GPIO_Init(GPIOB, &GPIO_InitStruct);				// now all the values are passed to the GPIO_Init() function which sets the GPIO registers

  GPIO_PinAFConfig(GPIOB, GPIO_PinSource6, GPIO_AF_USART1); //
  GPIO_PinAFConfig(GPIOB, GPIO_PinSource7, GPIO_AF_USART1);

  /* Now the USART_InitStruct is used to define the 
  * properties of USART1 
  */
  USART_InitStruct.USART_BaudRate = 115200;				// the baudrate is set to the value we passed into this init function
  USART_InitStruct.USART_WordLength = USART_WordLength_8b;// we want the data frame size to be 8 bits (standard)
  USART_InitStruct.USART_StopBits = USART_StopBits_1;		// we want 1 stop bit (standard)
  USART_InitStruct.USART_Parity = USART_Parity_No;		// we don't want a parity bit (standard)
  USART_InitStruct.USART_HardwareFlowControl = USART_HardwareFlowControl_None; // we don't want flow control (standard)
  USART_InitStruct.USART_Mode = USART_Mode_Tx | USART_Mode_Rx; // we want to enable the transmitter and the receiver
  USART_Init(USART1, &USART_InitStruct);					// again all the properties are passed to the USART_Init function which takes care of all the bit setting

  /* Here the USART1 receive interrupt is enabled
  * and the interrupt controller is configured 
  * to jump to the USART1_IRQHandler() function
  * if the USART1 receive interrupt occurs
  */
  USART_ITConfig(USART1, USART_IT_RXNE, ENABLE); // enable the USART1 receive interrupt
  // finally this enables the complete USART1 peripheral
  USART_Cmd(USART1, ENABLE);
}

/*
 * CONFIGURAR UART6
*/
void CONFI_UART6(){
  /* USARTx configured as follow:
        - BaudRate = 9600 baud  
        - Word Length = 8 Bits
        - One Stop Bit
        - No parity
        - Hardware flow control disabled (RTS and CTS signals)
        - Receive and transmit enabled
  */
  USART_InitStructure.USART_BaudRate = 9600;
  USART_InitStructure.USART_WordLength = USART_WordLength_8b;
  USART_InitStructure.USART_StopBits = USART_StopBits_1;
  USART_InitStructure.USART_Parity = USART_Parity_No;
  USART_InitStructure.USART_HardwareFlowControl = USART_HardwareFlowControl_None;
  USART_InitStructure.USART_Mode = USART_Mode_Rx | USART_Mode_Tx;
  STM_EVAL_COMInit(COM1, &USART_InitStructure);
}

/*
 * ENVIA STRING TX6 - ENVIA PARA O ESP8266
*/
void enviaTX6(char envia[],int str_len,int timing_delay){
  for(int q=0;q<str_len;q++){
    printf("%c",envia[q]);
  }
  estado=estado+1;
  delay(timing_delay);
  estado=estado;
}

/*
 * RECEO DA STRING RX6 E COMPARAO COM UMA STRING PRETENDIDA
*/
int recebeRX6(char str_conf[],int len_conf){
  if((strncmp(ch,str_conf,len_conf)==0)){
    k=0;
    apagaRecebe();
    cont_erro=0;
    estado++;
  }
  cont_erro++;
  //depois de tentar verificar o comando,  5 vez volta a enviar novamente
  if(cont_erro==5){
    estado--;
  }
  //depois de tentar verificar o comando,  10 vez faz reset ao mdulo e comea o processo novamente
  if(cont_erro==10){
    enviaTX6("\n\rAT+RST\n\r",10,20000);  //20s
    k=0;
    apagaRecebe();
    cont_erro=0;
    estado=0;
  }
  return estado;
}


/*
 * ENVIA STRING TX1
*/
void USART_puts(USART_TypeDef* USARTx,volatile char *s){
  while(*s){
    USART_SendData(USART1, (uint8_t) *s);
    while(!(USARTx->SR&0x00000040));
    *s++;
  }
}

/*
 * INTERRUPT HANDLER IMPLEMENTATION
*/
void USART6_IRQHandler(void){
  while (USART_GetFlagStatus(EVAL_COM1, USART_FLAG_RXNE) == RESET);
  ch[k] = USART_ReceiveData(EVAL_COM1);
  k++;
  //verifica se tem algum cliente conectado (0,CONNECT\r\n)
  if((ch[0]>='0' || ch[0]<='9') && ch[1]==',' && ch[2]=='C' && ch[3]=='O' && ch[4]=='N' && ch[5]=='N' && ch[6]=='E' && ch[7]=='C' && ch[8]=='T' && ch[9]=='\r' && ch[10]=='\n'){
    k=0;
    apagaRecebe();
    connect=1;
  }
  if((ch[0]>='0' || ch[0]<='9') && ch[1]==',' && ch[2]=='C' && ch[3]=='L' && ch[4]=='O' && ch[5]=='S' && ch[6]=='E' && ch[7]=='D' && ch[8]=='\r' && ch[9]=='\n'){
    k=0;
    apagaRecebe();
    connect=0;
  }
}

int main(void){
  const int cMaxEnvia=800;
  char envia[800]={0};
  char *token;
  int idx=0;
  int rececaoConcluida=0;
  CONFIG_UART1();
  CONFI_UART6();
  CONFIG_LED();
  while (1){
    GPIOD->BSRRL |=((0x01<<15));
    delay(1000);
    GPIOD->BSRRH |=((0x01<<15));
    delay(1000);
    
    if(connect==1 && rececaoConcluida==0 && ch[k-1]=='\n' && ch[k-2]=='\r' && ch[k-3]=='K' && ch[k-4]=='O'){
      substring(ch,rx,25,strlen(ch),'H');
      k=0;
      apagaRecebe(); 
      rececaoConcluida=1;
    }
    if(rececaoConcluida==1){
      token=strtok(rx,",");
      while((idx<cMaxEnvia) && token!=NULL){
        envia[idx++]=strtol(token,NULL,16);
        token=strtok(NULL,",");
      }
      USART_puts(USART1,envia);
      idx=0;
      //receo do pedido de status
      if(rx[2]=='1' && rx[3]=='C' && rx[7]=='7' && rx[8]=='6'){
        apagaRecebe();
        for(int l=0;l<800;l++){
          k=0;
          rx[l]=0;
        }
        while(USART_GetFlagStatus(USART1,USART_FLAG_RXNE)==RESET);
        estado_imp=USART_ReceiveData(USART1);
        apagaRecebe();
        for(int l=0;l<800;l++){
          k=0;
          rx[l]=0;
        }
        estado_imp=estado_imp;
        if(estado_imp==16 || estado_imp==20)
          status=0;
        else
          status=1;
        enviaTX6("\n\rAT+CIPSEND=0,1\n\r",18,250);
        if(recebeRX6(">",0)){
          printf("%d",status);
          delay(1000);
          apagaRecebe();
          k=0;
          for(int l=0;l<800;l++){
            rx[l]=0;
            envia[l]=0;
          }
        }
      }
      else{
        apagaRecebe();
        for(int l=0;l<800;l++){
          k=0;
          rx[l]=0;
          envia[l]=0;
        }
      }
      rececaoConcluida=0;
    }
    
    switch(estado){
      case 0:
        enviaTX6("\n\rATE0\n\r",8,250);
        break;
      case 1:
        recebeRX6("\r\nOK\r\n",6);
        recebeRX6("ATE0\r\n\r\nOK\r\n",12);
        break;
      case 2:
        enviaTX6("\n\rAT\n\r",6,250);
        break;
      case 3:
        recebeRX6("\r\nOK\r\n",6);
        break;
      case 4:
        enviaTX6("\n\rAT+CWMODE=1\n\r",15,250);
        break;
      case 5:
        recebeRX6("\r\nOK\r\n",6);
        break;
      case 6:
        enviaTX6("\n\rAT+CIPSTA=\"192.168.0.1\"\n\r",27,250);
        break;
      case 7:
        recebeRX6("\r\nOK\r\n",6);
        break;
      case 8:
        enviaTX6("\n\rAT+CWJAP=\"SKILLMIND\",\"4D431E05DA\"\n\r",36,15000);  //espera 15s
        break;
      case 9:
        recebeRX6("\r\nOK\r\n",6);
        break;
      case 10:
        enviaTX6("\n\rAT+CIPMUX=1\n\r",15,250);
        break;
      case 11:
        recebeRX6("\r\nOK\r\n",6);
        break;
      case 12:
        enviaTX6("\n\rAT+CIPSERVER=1,80\n\r",21,250);
        break;
      case 13:
        recebeRX6("\r\nOK\r\n",6);
        break;
    }
  }
}

/**
  * @brief  Retargets the C library printf function to the USART.
  * @param  None
  * @retval None
  */
PUTCHAR_PROTOTYPE{
  /* Place your implementation of fputc here */
  /* e.g. write a character to the USART */
  USART_SendData(EVAL_COM1, (uint8_t) ch);
  /* Loop until the end of transmission */
  while (USART_GetFlagStatus(EVAL_COM1, USART_FLAG_TC) == RESET);
  return ch;
}

#ifdef  USE_FULL_ASSERT

/**
  * @brief  Reports the name of the source file and the source line number
  *         where the assert_param error has occurred.
  * @param  file: pointer to the source file name
  * @param  line: assert_param error line source number
  * @retval None
  */
void assert_failed(uint8_t* file, uint32_t line){ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

  /* Infinite loop */
  while (1)
  {
  }
}
#endif
/*********** Portions COPYRIGHT 2012 Embest Tech. Co., Ltd.*****END OF FILE****/
